#!/bin/bash
#
# Copyright (c) 2015, Intel Corporation
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of Intel Corporation nor the names of its contributors
#      may be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# login as 'stack' and go to home directory
cd /home/stack/

use_commit_id=

if [ $use_commit_id == "no" ] 
then
  # use latest devstack code, clone Devstack source first
  git clone https://github.com/openstack-dev/devstack.git
  sleep 1
  echo -e "done download devstack \n"
  echo
elif [ $use_commit_id == "yes" ]
then
  # create /opt/stack
  sudo mkdir /opt/stack
  sudo chown stack:stack /opt/stack
  sleep 1
  echo -e "done creating /opt/stack/ \n"

  # parse onps_commit_ids and clone the repos and checkout their branches/tags
  cd /opt/stack

  while read p; do
    if [[ ! $p == \#* ]]
    then
      if [[ "$p" =~ [^a-zA-Z0-9] ]]
      then
        IFS=' ' read -a token <<< "$p"
        git clone ${token[1]} ${token[0]}
        cd ${token[0]}
 
        if [ "${token[2]}" = "master" ]
        then
          git checkout ${token[3]}
        else
          git checkout -b ${token[2]} ${token[3]}
        fi

        cd /opt/stack
      fi
     fi
  done </home/stack/onps_commit_ids
  echo -e "done cloning the OpenStack repos \n"
else
  echo "no devstack clone - use_commit_id must be yes or no "
  echo -e "please change the value of use_commit_id in prepare_stack.sh before running it again \n"
  exit 1
fi

##############################################################################
# This workaround is necessary as tox does not seem to inherit the environment
# so if there is a proxy defined, the pip calls will fail for tempest

if [ ! -e /opt/stack/tempest/tox.ini ]
then
# create /opt/stack
sudo mkdir -p /opt/stack
sudo chown stack:stack /opt/stack
sleep 1

cd /opt/stack/
git clone git://git.openstack.org/openstack/tempest.git
fi

sed_cmd="s|install_command = pip|install_command = env"
run_sed_cmd="no"

if [ -n "$http_proxy" ]
then
   sed_cmd="$sed_cmd http_proxy=$http_proxy"
   run_sed_cmd="yes"
fi

if [ -n "$https_proxy" ]
then
   sed_cmd="$sed_cmd https_proxy=$https_proxy"
   run_sed_cmd="yes"
fi
sed_cmd="$sed_cmd pip|g"

if [ "$run_sed_cmd" == "yes" ]
then
   sed -i "$sed_cmd" /opt/stack/tempest/tox.ini
fi

##############################################################################

# make a copy of local.conf to /home/stack/devstack
cp /home/stack/local.conf /home/stack/devstack/local.conf
sleep 1

# run stack.sk to install Devstack
# Devstack stack.sh check OS version, but FC21 is not included, so directly use FC21 will fail 
# need to use FORCE=yes to run stack.sh acript
echo -e "start installing devstack - will take 8-10 minutes \n"
sleep 5
cd /home/stack/devstack/
git checkout stable/kilo
sleep 1
FORCE=yes ./stack.sh

echo -e "done installing devstack \n"
sleep 1


# add matched network port to the bridge
# first, determine whether this is a controller node or a compute node with regular ovs
# for ovs-dpdk node, openstack script will add port to the bridge
typeset -l node_type ovs_type
node_type=`grep node "local.conf" | cut -d" " -f2`
ovs_type=`grep OVS_TYPE "local.conf" | cut -d"=" -f2`

#  add interface to the bridge created by devstack 
if [ $node_type == "controller" ] || [ $ovs_type == "ovs" ]
then
  echo -e "adding interface to bridge \n"

  bridge=`grep OVS_PHYSICAL_BRIDGE "local.conf" | cut -d= -f2`
  port=`echo $bridge | cut -d- -f2`
  sleep 1

  sudo ovs-vsctl add-port $bridge $port
  sleep 1

  sudo ovs-vsctl list-ports $bridge
  echo -e "done ovs adding port to bridge \n"
fi

# modify database to allow more connections
if [ $node_type == "controller" ] 
then
  echo -e "modify database configuration \n"
  sudo sed -i '/pid-file=/a \max_connections=1000' /etc/my.cnf
  sudo systemctl restart mariadb
  echo -e "done modify database configuration \n"
  sleep 1
fi

# the firewall restarts after stacking and preventing VM from access VNC server.
# need to flash iptables to get around 
if [ $node_type == "compute" ] 
then
  echo -e "flash iptables \n"
  sudo iptables -F
  echo -e "done flash iptables \n"
  echo
  sleep 1
fi

echo -e "done - openstack node installation completed \n"
exit 1


